@@ -1,4 +1,4 @@
-# $Id: HTTP.pm 323 2008-04-20 00:29:46Z rcaputo $
+# $Id: HTTP.pm 334 2008-07-13 08:07:22Z rcaputo $
 
 package POE::Component::Client::HTTP;
 
@@ -11,7 +11,7 @@ use constant DEBUG      => 0;
 use constant DEBUG_DATA => 0;
 
 use vars qw($VERSION);
-$VERSION = '0.83';
+$VERSION = '0.85';
 
 use Carp qw(croak);
 use HTTP::Response;
@@ -84,11 +84,6 @@ while (my ($encoding, $filter) = each %te_possible_filters) {
 #  qw(x-bzip2 gzip x-gzip deflate compress chunked identity)
 #);
 
-# Set default accept encoding here, but DON'T SET later, if the
-# request is streaming.
-
-my $accept_encoding = Net::HTTP::Methods::zlib_ok() ? 'gzip' : '';
-
 my %supported_schemes = (
   http  => 1,
   https => 1,
@@ -263,17 +258,6 @@ sub _poco_weeble_request {
     POE::Component::Client::HTTP::RequestFactory->parse_proxy($proxy_override);
   }
 
-  # Add an Accept-Encoding header if streaming is not enabled and the
-  # header doesn't already exist.
-  if (
-    !$heap->{factory}->is_streaming and
-    !$heap->{factory}->max_response_size and # or for limited chunk sizes
-    !defined($http_request->header('Accept-Encoding')) and
-    length($accept_encoding)
-  ) {
-    $http_request->header('Accept-Encoding', $accept_encoding);
-  }
-
   my $request = $heap->{factory}->create_request(
     $http_request, $response_event, $tag, $progress_event,
     $proxy_override, $sender
@@ -290,20 +274,20 @@ sub _poco_weeble_request {
 
   my $cm_req_id;
   eval {
-      # get a connection from Client::Keepalive
-      $request->[REQ_CONN_ID] = $heap->{cm}->allocate(
-        scheme  => $request->scheme,
-        addr    => $request->host,
-        port    => $request->port,
-        context => $request->ID,
-        event   => 'got_connect_done',
-        @timeout,
-      );
+    # get a connection from Client::Keepalive
+    $request->[REQ_CONN_ID] = $heap->{cm}->allocate(
+      scheme  => $request->scheme,
+      addr    => $request->host,
+      port    => $request->port,
+      context => $request->ID,
+      event   => 'got_connect_done',
+      @timeout,
+    );
   };
   if ($@) {
-      delete $heap->{request}->{$request->ID};
-      # we can reach here for things like host being invalid.
-      $request->error(400, $@);
+    delete $heap->{request}->{$request->ID};
+    # we can reach here for things like host being invalid.
+    $request->error(400, $@);
   }
 }
 
@@ -318,10 +302,24 @@ sub _poco_weeble_connect_done {
   my $connection = $response->{'connection'};
   my $request_id = $response->{'context'};
 
+  # Can't handle connections if we're shut down.
+  # TODO - How do we still get these?  Were they previously queued or
+  # something?
+  if ($heap->{is_shut_down}) {
+    _internal_cancel(
+      $heap, $request_id, 408, "Request timed out (request canceled)"
+    );
+    return;
+  }
+
   if (defined $connection) {
     DEBUG and warn "CON: request $request_id connected ok...";
 
     my $request = $heap->{request}->{$request_id};
+    unless (defined $request) {
+      DEBUG and warn "CON: ignoring connection for canceled request";
+      return;
+    }
 
     my $block_size = $heap->{factory}->block_size;
 
@@ -482,8 +480,9 @@ sub _poco_weeble_io_error {
   my ($kernel, $heap, $operation, $errnum, $errstr, $wheel_id) =
     @_[KERNEL, HEAP, ARG0..ARG3];
 
-  DEBUG and
-    warn "I/O: wheel $wheel_id encountered $operation error $errnum: $errstr";
+  DEBUG and warn(
+    "I/O: wheel $wheel_id encountered $operation error $errnum: $errstr"
+  );
 
   # Drop the wheel.
   my $request_id = delete $heap->{wheel_to_request}->{$wheel_id};
@@ -560,7 +559,7 @@ sub _poco_weeble_io_error {
     # not acceptable according to the accept headers sent in the
     # request.
 
-    $request->error (406, "Response larger than MaxSize - $request_id");
+    $request->error(406, "Response larger than MaxSize - $request_id");
   }
 }
 
@@ -911,12 +910,12 @@ sub _internal_cancel {
   my $request = delete $heap->{request}{$request_id};
   return unless defined $request;
 
-  DEBUG and warn "SHT: Shutdown is canceling request $request_id";
+  DEBUG and warn "CXL: canceling request $request_id";
   $request->remove_timeout();
 
   if (my $wheel = $request->wheel) {
     my $wheel_id = $wheel->ID;
-    DEBUG and warn "SHT: Request $request_id canceling wheel $wheel_id";
+    DEBUG and warn "CXL: Request $request_id canceling wheel $wheel_id";
     delete $heap->{wheel_to_request}{$wheel_id};
     delete $heap->{request_to_id}{$request->[REQ_REQUEST]};
     $wheel = undef;
@@ -933,7 +932,7 @@ sub _internal_cancel {
   }
 
   unless ($request->[REQ_STATE] & RS_POSTED) {
-    $request->error(408, "Request timed out (component shut down)");
+    $request->error($code, $message);
   }
 }
 
@@ -952,7 +951,7 @@ sub _poco_weeble_shutdown {
 
   # Shut down the connection manager subcomponent.
   if (defined $heap->{cm}) {
-    DEBUG and warn "SHT: Client::HTTP shutting down Client::Keepalive";
+    DEBUG and warn "CXL: Client::HTTP shutting down Client::Keepalive";
     $heap->{cm}->shutdown();
     delete $heap->{cm};
   }
@@ -1348,6 +1347,37 @@ to set the Content-Length header correctly.  Example:
   
   $kernel->post( ua => request, 'response', $request );
 
+=head1 CONTENT ENCODING AND COMPRESSION
+
+Transparent content decoding has been disabled as of version 0.84.
+This also removes support for transparent gzip requesting and
+decompression.
+
+To re-enable gzip compression, specify the gzip Content-Encoding and
+use HTTP::Response's decoded_content() method rather than content():
+
+  my $request = HTTP::Request->new(
+    GET => "http://www.yahoo.com/", [
+      'Accept-Encoding' => 'gzip'
+    ]
+  );
+
+  # ... time passes ...
+  
+  my $content = $response->decoded_content();
+
+The change in POE::Component::Client::HTTP behavior was prompted by
+changes in HTTP::Response that surfaced a bug in the component's
+transparent gzip handling.
+
+Allowing the application to specify and handle content encodings seems
+to be the most reliable and flexible resolution.
+
+For more information about the problem and discussions regarding the
+solution, see:
+L<http://www.perlmonks.org/?node_id=683833> and
+L<http://rt.cpan.org/Ticket/Display.html?id=35538>
+
 =head1 ENVIRONMENT
 
 POE::Component::Client::HTTP uses two standard environment variables:
